package net.peachjean.guice.resteasy;

import com.google.common.collect.Maps;
import com.google.inject.servlet.ServletModule;
import org.jboss.resteasy.logging.Logger;
import org.jboss.resteasy.plugins.server.servlet.ConfigurationBootstrap;
import org.jboss.resteasy.plugins.server.servlet.HttpServletDispatcher;

import java.util.*;

public class RestEasyModule extends ServletModule {
	private final static Logger logger = Logger.getLogger(RestEasyModule.class);

	private List<String> patterns = new ArrayList<String>();
	private List<String> regexes = new ArrayList<String>();

	private Map<String, String> parameters = Maps.newHashMap();
	private Map<String, String> initParameters = Maps.newHashMap();

	public RestEasyModule() {
	}

	public RestEasyModule handleRegex(String... regexes) {
		this.regexes.addAll(Arrays.asList(regexes));
		return this;
	}

	public RestEasyModule handle(String... patterns) {
		this.patterns.addAll(Arrays.asList(patterns));
		return this;
	}

	public RestEasyModule withParameter(String name, String value) {
		this.parameters.put(name, value);
		return this;
	}

	public RestEasyModule withInitParameter(String name, String value) {
		this.initParameters.put(name, value);
		return this;
	}

	@Override
	protected void configureServlets() {
		bind(SimpleGuiceResteasyBootstrap.class).asEagerSingleton();
		bind(GuiceRestEasyDispatcher.class).asEagerSingleton();
		bind(ConfigurationBootstrap.class).toInstance(new GuiceResteasyConfigurationBootstrap(
				Collections.unmodifiableMap(parameters),
				Collections.unmodifiableMap(initParameters)));

		for (String path : this.patterns) {
			serve(path).with(GuiceRestEasyDispatcher.class);
		}
		for (String regex : this.regexes) {
			serveRegex(regex).with(GuiceRestEasyDispatcher.class);
		}
	}
}
