package net.peachjean.guice.resteasy;

import com.google.inject.Binding;
import com.google.inject.Inject;
import com.google.inject.Injector;
import org.jboss.resteasy.logging.Logger;
import org.jboss.resteasy.plugins.guice.GuiceResourceFactory;
import org.jboss.resteasy.plugins.server.servlet.ResteasyBootstrap;
import org.jboss.resteasy.spi.Registry;
import org.jboss.resteasy.spi.ResourceFactory;
import org.jboss.resteasy.spi.ResteasyProviderFactory;
import org.jboss.resteasy.util.GetRestful;

import javax.servlet.ServletContext;
import javax.servlet.ServletContextEvent;
import javax.ws.rs.ext.Provider;
import java.lang.reflect.Type;

public class SimpleGuiceResteasyBootstrap extends ResteasyBootstrap {
	private final static Logger logger = Logger.getLogger(SimpleGuiceResteasyBootstrap.class);

	private Injector injector;

	@Inject
	public SimpleGuiceResteasyBootstrap(Injector injector) {
		this.injector = injector;
	}

	@Override
	public void contextInitialized(ServletContextEvent event) {
		super.contextInitialized(event);
		final ServletContext context = event.getServletContext();
		final Registry registry = (Registry) context.getAttribute(Registry.class.getName());
		final ResteasyProviderFactory providerFactory = (ResteasyProviderFactory) context.getAttribute(ResteasyProviderFactory.class.getName());

		for (final Binding<?> binding : injector.getBindings().values()) {
			final Type type = binding.getKey().getTypeLiteral().getType();
			if (type instanceof Class) {
				final Class<?> beanClass = (Class) type;
				if (GetRestful.isRootResource(beanClass)) {
					final ResourceFactory resourceFactory = new GuiceResourceFactory(binding.getProvider(), beanClass);
					logger.info("registering factory for {0}", beanClass.getName());
					registry.addResourceFactory(resourceFactory);
				}
				if (beanClass.isAnnotationPresent(Provider.class)) {
					logger.info("registering provider instance for {0}", beanClass.getName());
					providerFactory.registerProviderInstance(binding.getProvider().get());
				}
			}
		}
	}
}
